import 'package:parse_server_sdk_flutter/parse_server_sdk.dart';

class LogEventType {
  late String? objectId;
  late String value;
  late bool hasEndTime;
  late int? defaultReminderDuration;
  late String? notes;

  LogEventType(ParseObject object) {
    objectId = object.get<String>('objectId');
    value = object.get<String>('value')!;
    hasEndTime = object.get<bool>('hasEndTime')!;
    defaultReminderDuration = object.get<num>('defaultReminderDuration') != null
        ? object.get<num>('defaultReminderDuration')!.toInt()
        : null;
    notes = object.get<String>('notes');
  }

  static Future<List<LogEventType>> fetchAll() async {
    QueryBuilder<ParseObject> query =
        QueryBuilder<ParseObject>(ParseObject('LogEventType'));
    final ParseResponse apiResponse = await query.query();

    if (apiResponse.success && apiResponse.results != null) {
      return apiResponse.results!
          .map((e) => LogEventType(e as ParseObject))
          .toList();
    } else {
      return [];
    }
  }

  static Future<LogEventType?> get(String objectId) async {
    QueryBuilder<ParseObject> query =
        QueryBuilder<ParseObject>(ParseObject('LogEventType'))
          ..whereEqualTo('objectId', objectId);
    final ParseResponse apiResponse = await query.query();

    if (apiResponse.success && apiResponse.results != null) {
      return LogEventType(apiResponse.result.first);
    }
  }

  static Future<void> save({
    required String value,
    required bool hasEndTime,
    int? defaultReminderDuration,
    String? notes,
  }) async {
    final logEventType = ParseObject('LogEventType')
      ..set('value', value)
      ..set('hasEndTime', hasEndTime)
      ..set('defaultReminderDuration', defaultReminderDuration)
      ..set('notes', notes);
    await logEventType.save();
  }

  static Future<void> update(
    String objectId, {
    String? value,
    bool? hasEndTime,
    int? defaultReminderDuration,
    String? notes,
  }) async {
    var logEventType = ParseObject('LogEventType')..objectId = objectId;
    if (value != null) {
      logEventType.set('value', value);
    }
    if (hasEndTime != null) {
      logEventType.set('hasEndTime', hasEndTime);
    }
    if (defaultReminderDuration != null) {
      logEventType.set('defaultReminderDuration', defaultReminderDuration);
    }
    if (notes != null) {
      logEventType.set('notes', notes);
    }
    await logEventType.save();
  }

  Future<void> delete() async {
    var logEventType = ParseObject('LogEventType')..objectId = objectId;
    await logEventType.delete();
  }
}
