import 'package:diameter/components/dialogs.dart';
import 'package:diameter/components/progress_indicator.dart';
import 'package:diameter/config.dart';
import 'package:diameter/navigation.dart';
import 'package:diameter/screens/meal/meal_portion_type_detail.dart';
import 'package:flutter/material.dart';
import 'package:diameter/models/meal_portion_type.dart';

class MealPortionTypeListScreen extends StatefulWidget {
  static const String routeName = '/meal-portion-types';

  const MealPortionTypeListScreen({Key? key}) : super(key: key);

  @override
  _MealPortionTypeListScreenState createState() =>
      _MealPortionTypeListScreenState();
}

class _MealPortionTypeListScreenState extends State<MealPortionTypeListScreen> {
  late Future<List<MealPortionType>?> _mealPortionTypes;

  void refresh({String? message}) {
    setState(() {
      _mealPortionTypes = MealPortionType.fetchAll();
    });
    setState(() {
      if (message != null) {
        var snackBar = SnackBar(
          content: Text(message),
          duration: const Duration(seconds: 2),
        );
        ScaffoldMessenger.of(context)
          ..removeCurrentSnackBar()
          ..showSnackBar(snackBar);
      }
    });
  }

  void onDelete(MealPortionType mealPortionType) {
    mealPortionType
        .delete()
        .then((_) => refresh(message: 'Meal Portion Type deleted'));
  }

  void handleDeleteAction(MealPortionType mealPortionType) async {
    if (showConfirmationDialogOnDelete) {
      Dialogs.showConfirmationDialog(
        context: context,
        onConfirm: () => onDelete(mealPortionType),
        message: 'Are you sure you want to delete this Meal Portion Type?',
      );
    } else {
      onDelete(mealPortionType);
    }
  }

  @override
  void initState() {
    super.initState();
    refresh();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: const Text('Meal Portion Types'),
        actions: <Widget>[
          IconButton(onPressed: refresh, icon: const Icon(Icons.refresh))
        ],
      ),
      drawer: const Navigation(
          currentLocation: MealPortionTypeListScreen.routeName),
      body: Column(
        mainAxisAlignment: MainAxisAlignment.center,
        children: <Widget>[
          Expanded(
            child: FutureBuilder<List<MealPortionType>?>(
              future: _mealPortionTypes,
              builder: (context, snapshot) {
                return ViewWithProgressIndicator(
                    snapshot: snapshot,
                    child: snapshot.data == null || snapshot.data!.isEmpty
                        ? Row(
                            mainAxisAlignment: MainAxisAlignment.center,
                            children: const [
                              Padding(
                                padding: EdgeInsets.all(10.0),
                                child: Text('No Meal Portion Types'),
                              )
                            ],
                          )
                        : ListView.builder(
                            padding: const EdgeInsets.only(top: 10.0),
                            itemCount: snapshot.data != null
                                ? snapshot.data!.length
                                : 0,
                            itemBuilder: (context, index) {
                              final mealPortionType = snapshot.data![index];
                              return ListTile(
                                onTap: () {
                                  Navigator.push(
                                    context,
                                    MaterialPageRoute(
                                      builder: (context) =>
                                          MealPortionTypeDetailScreen(
                                        mealPortionType: mealPortionType,
                                      ),
                                    ),
                                  ).then(
                                      (message) => refresh(message: message));
                                },
                                title: Text(mealPortionType.value),
                                subtitle: Text(mealPortionType.notes ?? ''),
                                trailing: Row(
                                  mainAxisSize: MainAxisSize.min,
                                  children: [
                                    IconButton(
                                      icon: const Icon(
                                        Icons.delete,
                                        color: Colors.blue,
                                      ),
                                      onPressed: () =>
                                          handleDeleteAction(mealPortionType),
                                    ),
                                  ],
                                ),
                              );
                            }));
              },
            ),
          ),
        ],
      ),
      floatingActionButton: FloatingActionButton(
        onPressed: () {
          Navigator.push(
            context,
            MaterialPageRoute(
              builder: (context) => const MealPortionTypeDetailScreen(),
            ),
          ).then((message) => refresh(message: message));
        },
        child: const Icon(Icons.add),
      ),
    );
  }
}
